/**
  *  \file server/filenotify.cc
  *  \brief File Server Notification
  *
  *  The router needs a way to notify the file server of changes, so
  *  it can update its state-of-the-world. This is done by using the
  *  file server's FORGET command, using the parameter passed to a
  *  session as "-WDIR=...".
  *
  *  The file server has been extended to support multiple concurrent
  *  connections. One is reserved for the router, which has this
  *  connection open permanently. When the router performs some
  *  operation that requires notification of the file server, this
  *  gives the convenient property that the notification has been
  *  correctly executed by the time the router operation returns.
  *
  *  If you're using the router without the file server, configure
  *  File.Host as empty in the configuration file.
  *
  *  \todo Automatically reconnect upon losing connection (e.g. if
  *  file server is restarted).
  */

#include <list>
#include <utility>
#include <semaphore.h>
#include <pthread.h>
#include <stdio.h>
#include "server/filenotify.h"
#include "server/net.h"
#include "cpluslib/format.h"
#include "server/request.h"

/* File server parameters */
static string_t arg_file_host = "127.0.0.1";
static string_t arg_file_port = "9998";

/* Notify parameters */
static int gFileServerFD = -1;

static void
fullWrite(int fd, const char* data, std::size_t n)
{
    while (n > 0) {
        int did = write(fd, data, n);
        if (did < 0) {
            perror("write");
            break;
        }
        data += did;
        n -= did;
    }
}

/** Skip reply from file server. This only reads inline success/
    failure replies, no structured (bulk/multi-bulk) replies, because
    it's only intended to handle the result of FORGET.
    \retval true success reply received
    \retval false invalid or error reply received */
static bool
skipReply(int fd)
{
    Request req;

    /* Read a line */
    const char* line;
    while ((line = req.readLine()) == 0) {
        if (req.readMore(fd) != Request::sSuccess && !req.shift()) {
            printf("error: invalid server reply\n");
            return false;
        }
    }

    if (*line == '+' || *line == ':') {
        /* in-line string, success */
        return true;
    } else {
        /* error */
        printf("error: invalid server reply '%s'\n", line);
        return false;
    }
}

/** Request notification. Places the cmd/arg pair in the queue which
    is asynchronously transmitted.
    \param cmd command ("FORGET")
    \param arg argument (directory name) */
void
notifyFileServer(string_t cmd, string_t arg)
{
    if (gFileServerFD >= 0) {
        string_t fullCmd = format("*2\r\n"
                                  "$%d\r\n"
                                  "%s\r\n"
                                  "$%d\r\n"
                                  "%s\r\n",
                                  cmd.size(), cmd,
                                  arg.size(), arg);
        fullWrite(gFileServerFD, fullCmd.data(), fullCmd.size());
        skipReply(gFileServerFD);
    }
}

/** Handle file server configuration items. */
void
handleFileServerConfig(string_t key, string_t value)
{
    if (key == "FILE.HOST") {
        arg_file_host = value;
    } else if (key == "FILE.PORT") {
        arg_file_port = value;
    }
}

/** Initialize file server notification. Creates objects and the
    background thread.
    \retval true success
    \retval false unable to create some resources */
bool
initNotifyFileServer()
{
    int loops = 0;
    if (arg_file_host.empty()) {
        /* No notification configured */
        return true;
    }
    while (1) {
        /* Try to connect to file server for some seconds */
        gFileServerFD = Connect().withAddress(arg_file_host, arg_file_port);
        if (gFileServerFD >= 0) {
            return true;
        }
        if (++loops > 5) {
            perror(arg_file_host.c_str());
            return false;
        }
        sleep(1);
    }
}
