/**
  *  \file server/jsonout.cc
  *  \brief Server: JSON output
  */

#include "server/jsonout.h"
#include "server/serverio.h"
#include "cpluslib/format.h"

JsonWriter::JsonWriter()
    : need_comma_before_next_obj(false)
{ }

void
JsonWriter::flushComma()
{
    if (need_comma_before_next_obj) {
        line += ",";
        if (line.size() > 80) {
            writeClient(line);
            line.clear();
        }
        need_comma_before_next_obj = false;
    }
}

void
JsonWriter::startArray(string_t name)
{
    flushComma();
    line += "\"";
    line += name;
    line += "\":[";
}

void
JsonWriter::startArray()
{
    flushComma();
    line += "[";
}

void
JsonWriter::endArray()
{
    line += "]";
    need_comma_before_next_obj = true;
}

void
JsonWriter::writeString(const string_t value)
{
    flushComma();
    line += "\"";
    for (string_t::size_type i = 0; i < value.size(); ++i) {
        uint8_t ch = value[i];
        if (ch == '\\') {
            line += "\\\\";
        } else if (ch == '"') {
            line += "\\\"";
        } else if (ch >= 32) {
            line += char(ch);
        } else if (ch == '\n') {
            line += "\\n";
        } else if (ch == '\r') {
            line += "\\r";
        } else {
            line += format("\\u%04X", ch);
        }
    }
    line += "\"";
    need_comma_before_next_obj = true;
}

void
JsonWriter::writeInt(int32_t value)
{
    flushComma();
    line += format("%d", value);
    need_comma_before_next_obj = true;
}

void
JsonWriter::writeNull()
{
    flushComma();
    line += "null";
    need_comma_before_next_obj = true;
}

void
JsonWriter::startObject(string_t name)
{
    flushComma();
    line += "\"";
    line += name;
    line += "\":{";
}

void
JsonWriter::startObject()
{
    flushComma();
    line += "{";
}

void
JsonWriter::endObject()
{
    line += "}";
    need_comma_before_next_obj = true;
}

void
JsonWriter::writeString(string_t name, string_t value)
{
    flushComma();
    line += "\"";
    line += name;
    line += "\":";
    writeString(value);
}

void
JsonWriter::writeInt(string_t name, int32_t value)
{
    flushComma();
    line += "\"";
    line += name;
    line += "\":";
    writeInt(value);
}

void
JsonWriter::writeNull(string_t name)
{
    flushComma();
    line += "\"";
    line += name;
    line += "\":";
    writeNull();
}

void
JsonWriter::start()
{
    line = "{";
    need_comma_before_next_obj = false;
}

void
JsonWriter::end()
{
    writeClient(line + "}");
    line.clear();
}
